#include <assert>
#include <string>
#include <iostream>
using namespace std;

#define BitsIn(T) (8*sizeof(T))

class BitString {
public: typedef int size_type;
private:
  typedef unsigned char byte;
  byte *mItems;
  size_type mSize, mCapacity;//both measured in bits, not bytes

  /*this class exists so I can write Bit methods for this class instead of
   for every type this class can be constructed with, i.e., I don't need an
   = operator in class Bit for bool, char, or int because of the one for BitValue*/
  class BitValue {
  private:
    bool mValue;
    
  public:
    BitValue(bool value) : mValue(value) {}
    BitValue(char value) : mValue(value != '0') {}
    BitValue(int value) : mValue(value != 0) {}
    bool IsSet() const {return mValue;}
  };
  
  class Bit {
  private:
    const BitString* Owner;
    
    //non-members can set and clear Bit by assigning it true or false
    void Set(bool value) {
      if (value)
        Owner->mItems[ByteIndex] |= BitMask(ByteOffset);//set bit
      else
        Owner->mItems[ByteIndex] &= ~BitMask(ByteOffset);//clear bit
    }
    
  public:
    size_type ByteIndex;
    size_type ByteOffset;
    Bit(size_type offset, const BitString* owner)
      : Owner(owner), ByteIndex(offset / BitsIn(byte)), ByteOffset(offset % BitsIn(byte)) {}
    
    bool IsSet() const {
      return (Owner->mItems[ByteIndex] & BitMask(ByteOffset)) != 0;
    }

    bool operator==(const Bit& rhs) const {return IsSet() == rhs.IsSet();}
    bool operator==(const BitValue& rhs) const {return IsSet() == rhs.IsSet();}
    bool operator!=(const Bit& rhs) const {return IsSet() != rhs.IsSet();}
    bool operator!=(const BitValue& rhs) const {return IsSet() != rhs.IsSet();}
    Bit& operator=(const Bit& rhs) {Set(rhs.IsSet()); return *this;}
    Bit& operator=(const BitValue& rhs) {Set(rhs.IsSet()); return *this;}
    Bit& operator&=(const Bit& rhs) {Set(IsSet() && rhs.IsSet()); return *this;}
    Bit& operator|=(const Bit& rhs) {Set(IsSet() || rhs.IsSet()); return *this;}
    Bit& operator^=(const Bit& rhs) {Set(IsSet() != rhs.IsSet()); return *this;}
    Bit& operator&=(const BitValue& rhs) {Set(IsSet() && rhs.IsSet()); return *this;}
    Bit& operator|=(const BitValue& rhs) {Set(IsSet() || rhs.IsSet()); return *this;}
    Bit& operator^=(const BitValue& rhs) {Set(IsSet() != rhs.IsSet()); return *this;}
    bool operator&(const Bit& rhs) const {return IsSet() && rhs.IsSet();}
    bool operator|(const Bit& rhs) const {return IsSet() || rhs.IsSet();}
    bool operator^(const Bit& rhs) const {return IsSet() != rhs.IsSet();}
    bool operator&(const BitValue& rhs) const {return IsSet() && rhs.IsSet();}
    bool operator|(const BitValue& rhs) const {return IsSet() || rhs.IsSet();}
    bool operator^(const BitValue& rhs) const {return IsSet() != rhs.IsSet();}
    bool operator~() const {return !IsSet();}
    void Invert() {Set(!IsSet());}
  };
  
public:
  //constructors and destructors
  BitString(size_type size = 0, bool init = false) : mItems(0), mCapacity(0) {
    Resize(size); if (init) SetAll(false);}
  BitString(const BitString &rhs) : mItems(0), mCapacity(0) {*this = rhs;}
  BitString(char *str) : mItems(0), mCapacity(0) {*this = string(str);}
  BitString(const string &str);
  ~BitString() {delete [] mItems;}

  //operators
  bool operator==(const BitString &rhs) const;
  bool operator!=(const BitString &rhs) const {return !(*this == rhs);}
  bool operator<(const BitString &rhs) const;
  bool operator<=(const BitString &rhs) const;
  bool operator>(const BitString &rhs) const {return rhs < *this;}
  bool operator>=(const BitString &rhs) const {return rhs <= *this;}

  BitString& operator=(const BitString &rhs);

  Bit operator[](size_type offset) {AssertValid(offset); return Bit(offset, this);}
  const Bit operator[](size_type offset) const {AssertValid(offset); return Bit(offset, this);}

  BitString& operator<<=(int amount);
  BitString& operator>>=(int amount);
  BitString operator<<(int amount) const;  
  BitString operator>>(int amount) const;  

  BitString& operator&=(const BitString &rhs);
  BitString& operator|=(const BitString &rhs);
  BitString& operator^=(const BitString &rhs);
  BitString operator&(const BitString &rhs) const;
  BitString operator|(const BitString &rhs) const;
  BitString operator^(const BitString &rhs) const;
  BitString operator~() const;

  //inline methods
  void AssertValid(size_type offset) const {assert(offset >= 0 && offset < mSize);}
  //returns a byte where only bit at byteOffset is set
  static byte BitMask(int byteOffset) {return 1 << (BitsIn(byte) - byteOffset - 1);}
  size_type Size() const {return mSize;}
  static size_type BytesNeeded(size_type bits) {return (bits + BitsIn(byte) - 1) / BitsIn(byte);}
  size_type BytesOccupied() const {return BytesNeeded(mCapacity);}
  size_type BytesUsed() const {return BytesNeeded(mSize);}
  size_type Capacity() const {return mCapacity;}

  //non-inline methods
  void Grow(size_type size);
  void Invert();
  void Pack();
  ostream& Print(ostream& out) const;
  void Resize(size_type size);
  void Reverse();
  void SetAll(bool value);

  //friendships
  friend Bit;
  friend int ord(const Bit &bit);
};

//*** constructors ***
BitString::BitString(const string &str) : mItems(0), mCapacity(0) {
  Resize(str.size());
  for (size_type i = 0; i < mSize; ++i)
    (*this)[i] = str[i];
}

//*** operators ***
BitString& BitString::operator=(const BitString &rhs) {
  if (BytesOccupied() < rhs.BytesUsed()) Resize(rhs.mSize);

  mSize = rhs.mSize;
  for (size_type i = 0; i < BytesUsed(); ++i)
    mItems[i] = rhs.mItems[i];

  return *this;
}

bool BitString::operator==(const BitString &rhs) const {
  size_type i, minBytesUsed = min(BytesUsed(), rhs.BytesUsed());
  for (i = 0; i < minBytesUsed - 1; ++i)
    if (mItems[i] != rhs.mItems[i]) return false;
  
  size_type minSize = min(mSize, rhs.mSize);
  for (i = (minBytesUsed - 1)*BitsIn(byte); i < minSize; ++i)
    if ((*this)[i] != rhs[i]) return false;
  
  /*bigger BitString must be all 0's to return true;
    note, this could be optimized by using mItems also*/
  if (mSize > rhs.mSize) {
    for (i = minSize; i < mSize; ++i)
      if ((*this)[i] == true) return false;
  } else if (rhs.mSize > mSize)
    for (i = minSize; i < rhs.mSize; ++i)
      if (rhs[i] == true) return false;
      
  return true;
}

bool BitString::operator<(const BitString &rhs) const {
  size_type minSize = min(mSize, rhs.mSize);
  int comp;
  for (size_type i = 0; i < minSize; ++i) {
    comp = ord((*this)[i]) - ord(rhs[i]);
    if (comp == -1) return true;
    else if (comp == 1) return false;
    assert(comp == 0);
  }
  
  return mSize < rhs.mSize;
}

bool BitString::operator<=(const BitString &rhs) const {
  size_type minSize = min(mSize, rhs.mSize);
  int comp;
  for (size_type i = 0; i < minSize; ++i) {
    comp = ord((*this)[i]) - ord(rhs[i]);
    if (comp == -1) return true;
    else if (comp == 1) return false;
    assert(comp == 0);
  }
  
  return mSize <= rhs.mSize;
}

BitString& BitString::operator<<=(int amount) {
  if (amount < 0) return *this >>= -amount;
  if (amount > mSize) amount = mSize;
  size_type i;
  
  for (i = 0; i < mSize - amount; ++i)
    (*this)[i] = (*this)[i + amount];
  for (i = mSize - amount; i < mSize; ++i)
    (*this)[i] = false;
  
  return *this;
}

BitString& BitString::operator>>=(int amount) {
  if (amount < 0) return *this <<= -amount;
  if (amount > mSize) amount = mSize;
  size_type i;

  for (i = mSize - 1; i >= amount; --i)
    (*this)[i] = (*this)[i - amount];
  for (i = 0; i < amount; ++i)
    (*this)[i] = false;
    
  return *this;
}


BitString BitString::operator<<(int amount) const {
  BitString result = *this;
  return result <<= amount;
}  

BitString BitString::operator>>(int amount) const {
  BitString result = *this;
  return result >>= amount;
}  

BitString& BitString::operator&=(const BitString &rhs) {
  size_type minUsed = min(BytesUsed(), rhs.BytesUsed());
  for (size_type i = 0; i < minUsed; ++i)
    mItems[i] &= rhs.mItems[i];
  return *this;
}

BitString& BitString::operator|=(const BitString &rhs) {
  size_type minUsed = min(BytesUsed(), rhs.BytesUsed());
  for (size_type i = 0; i < minUsed; ++i)
    mItems[i] |= rhs.mItems[i];
  return *this;
}

BitString& BitString::operator^=(const BitString &rhs) {
  size_type minUsed = min(BytesUsed(), rhs.BytesUsed());
  for (size_type i = 0; i < minUsed; ++i)
    mItems[i] ^= rhs.mItems[i];
  return *this;
}

BitString BitString::operator&(const BitString &rhs) const {
  BitString result = *this;
  return result &= rhs;
}

BitString BitString::operator|(const BitString &rhs) const {
  BitString result = *this;
  return result |= rhs;
}

BitString BitString::operator^(const BitString &rhs) const {
  BitString result = *this;
  return result ^= rhs;
}

BitString BitString::operator~() const {
  BitString result(mSize);
  for (size_type i = 0; i < BytesUsed(); ++i)
    result.mItems[i] = ~mItems[i];
  return result;
}

//*** methods ***
/*Grow is like Resize, only the values already in this are not changed. Despite the name,
  Grow can be used to shrink the BitString*/
void BitString::Grow(size_type size) {
  if (size < mSize) {
    assert(size >= 0);
    mSize = size;
    return;
  }
  
  if (mCapacity >= size) {
    assert(size >= 0);
    mSize = size;
    return;
  }
  
  BitString temp = *this;
  Resize(size);
  *this = temp;
}

void BitString::Invert() {
  for (size_type i = 0; i < BytesUsed(); ++i)
    mItems[i] = ~mItems[i];
}

//Resizes mItems so mCapacity = BytesUsed(). mItems's values are preserved.
void BitString::Pack() {
  if (BytesOccupied() == BytesUsed()) return;
  assert(BytesUsed() != 0);//should be true since BytesOccupied() != BytesUsed()
  byte *newItems = new byte[BytesUsed()];
  
  for (size_type i = 0; i < BytesUsed(); ++i)
    newItems[i] = mItems[i];
    
  delete [] mItems;
  mItems = newItems;
  mCapacity = BitsIn(byte)*BytesUsed();
}

ostream& BitString::Print(ostream& out) const {
  for (size_type i = 0; i < mSize; ++i)
    out << ((*this)[i].IsSet() ? '1' : '0');

  return out;
}

/*Changes mSize to size by reallocating mItems, so mItems's values are not preserved.
  Use Grow if you want the values in mItems to be preserved.*/
void BitString::Resize(size_type size) {
  if (BytesNeeded(size) <= BytesOccupied()) {
    mSize = size;
    return;
  }
  
  delete [] mItems;
  mSize = mCapacity = size;
  mItems = size > 0 ? new byte[BytesOccupied()] : 0;
}

void BitString::Reverse() {
  bool temp;
  for (size_type i = 0; i < mSize / 2; ++i) {
    temp = (*this)[i].IsSet();
    (*this)[i] = (*this)[mSize - i - 1];
    (*this)[mSize - i - 1] = temp;
  }
}

void BitString::SetAll(bool value) {
  for (size_type i = 0; i < BytesUsed(); ++i)
    mItems[i] = value ? 0xFF : 0;
}

//friend functions
inline int ord(const BitString::Bit &bit) {return bit == true ? 1 : 0;}

//non-member functions
inline ostream& operator<<(ostream& out, const BitString &bs) {
  return bs.Print(out);
}

/*I could have used enum Endian {Little, Big}, GetEndiand, and SetEndian in
  BitString, but this is more object-oriented*/
class Endian {
public: enum Type {Little, Big};
private: Type mType;
public:
  Endian(Type type = Little) {mType = type;}
  bool operator==(const Endian &rhs) const {return mType == rhs.mType;}
  bool operator!=(const Endian &rhs) const {return !(*this == rhs);}
  bool operator==(Type rhs) const {return mType == rhs;}
  bool operator!=(Type rhs) const {return !(*this == rhs);}
  Type GetType() const {return mType;}
};

template<class Num>
class BinaryNumber : public BitString {
private:
  void Assign(Num num);
  
public:
  Endian mEndian;
  BinaryNumber(Num num = 0, Endian endian = Endian::Little)
    : BitString(sizeof(Num)), mEndian(endian) {Assign(num);}

  void AssignTo(Num &num) const;

  template<class NumT2>  
  BinaryNumber<Num>& operator=(const BinaryNumber<NumT2> &rhs) {
    NumT2 num;
    rhs.AssignTo(num);
    Assign(Num(num));
    return *this;
  }

  template<class NumT2>  
  bool operator<(const BinaryNumber<NumT2> &rhs) {
    Num lhsNum;
    AssignTo(lhsNum);

    NumT2 rhsNum;
    rhs.AssignTo(rhsNum);

    return lhsNum < rhsNum;
  }

  template<class NumT2>  
  bool operator>(const BinaryNumber<NumT2> &rhs) {return rhs < *this;}

  template<class NumT2>  
  bool operator<=(const BinaryNumber<NumT2> &rhs) {
    Num lhsNum;
    AssignTo(lhsNum);

    NumT2 rhsNum;
    rhs.AssignTo(rhsNum);

    return lhsNum <= rhsNum;
  }
  
  template<class NumT2>  
  bool operator>=(const BinaryNumber<NumT2> &rhs) {return rhs <= *this;}
};

template<class Num>
void BinaryNumber<Num>::Assign(Num num) {
  Resize(BitsIn(Num));
    
  if (mEndian == Endian::Little)
    for (size_type i = 0; i < Size(); ++i, num >>= 1)
      (*this)[i] = num % 2 != 0;
  else
    for (size_type i = 0; i < Size(); ++i, num >>= 1)
      (*this)[Size() - i - 1] = num % 2 != 0;
}

template<class Num>
void BinaryNumber<Num>::AssignTo(Num &num) const {
  num = 0;
  
  if (mEndian == Endian::Big)
    for (size_type i = 0; i < Size(); ++i) {
      if ((*this)[i] == true) ++num;
      if (i < Size() - 1) num <<= 1;
    }
  else
    for (size_type i = 0; i < Size(); ++i) {
      if ((*this)[Size() - i - 1] == true) ++num;
      if (i < Size() - 1) num <<= 1;
    }
}